/*****************************************************************
 * Copyright (c) ParaSoft Corporation 1991.  All rights reserved.*
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF ParaSoft       *
 * The copyright notice above does not evidence any              *
 * actual or intended publication of such source code.           *
 *****************************************************************/

#include "extern.h"

/********************************** C H E C K ***************************************/

Node *check_InquireStmt (root)
Node *root;
{
	Node *list = list_invert (root->InquireStmt.IoSpecList);
	Node *nspec, *temp;
	struct S_IoSpec *spec;
	Unit *unit = unit_malloc ();
	char *name, *cons;
	int first;
	char *err_mess_1 = "cannot use both UNIT and FILE specifiers in INQUIRE statement";

	if (flag_cubix)
		cubix_io_called = 1;

	root->InquireStmt.unit = unit;

	if (root->InquireStmt.Lpeqi != NN) {
		root->InquireStmt.Lpeqi = NN;
		root->InquireStmt.Lp = &Lp;
	}

	first = 1;
	for (LIST_LOOP (nspec, list, temp)) {
		spec = &nspec->IoSpec;

		if (spec->Name == NN) {
			if (first) {
				if (unit->unit != NN) {
					io_err_spec_used ("UNIT");
					continue;
				}

				if (!IS_INTEGER (spec->Expr)) {
					io_err_spec_type ("UNIT" , "INTEGER");
					continue;
				}

				unit->unit = spec->Expr;
			}
			else
				fatal (
				"UNIT specifier with keyword omitted must be the first item in OPEN");

			continue;
		}

		name = U_strupper (spec->Name->com.str);

		if (spec->Times != NN) {
			sprintf (serr, "cannot use '*' with specifier %s", name);
			fatal (serr);
			continue;
		}

		if (io_spec_is ("UNIT", name, "INTEGER", spec, unit->unit)) {
			unit->unit = spec->Expr;

			if (unit->file != NN)
				fatal (err_mess_1);
		}
		else if (io_spec_is_name ("IOSTAT", name, "INTEGER", spec, unit->iostat)) {
			unit->iostat = spec->Expr;
		}
		else if (io_spec_is ("ERR", name, "INTEGER", spec, unit->err)) {
			if (spec->Expr->com.type != INTEGER_CONSTANT) {
				fatal ("specifier ERR needs a label"); 
				continue;
			}

			unit->err = spec->Expr;

			label_install_ref (spec->Expr);
		}
		else if (io_spec_is ("FILE", name, "CHARACTER", spec, unit->file)) {
			unit->file = spec->Expr;

			if (unit->unit != NN)
				fatal (err_mess_1);
		}
		else if (io_spec_is_name ("EXIST", name, "LOGICAL", spec, unit->exist)) {
			unit->exist = spec->Expr;
		}
		else if (io_spec_is_name ("OPENED", name, "LOGICAL", spec, unit->opened)) {
			unit->opened = spec->Expr;
		}
		else if (io_spec_is_name ("NUMBER", name, "INTEGER", spec, unit->number)) {
			unit->number = spec->Expr;
		}
		else if (io_spec_is_name ("NAMED", name, "LOGICAL", spec, unit->named)) {
			unit->named = spec->Expr;
		}
		else if (io_spec_is_name ("NAME", name, "CHARACTER", spec, unit->name)) {
			unit->name = spec->Expr;
		}
		else if (io_spec_is_name ("ACCESS", name, "CHARACTER", spec, unit->access)) {
			unit->access = spec->Expr;
		}
		else if (io_spec_is_name ("SEQUENTIAL", name, "CHARACTER", spec, unit->sequential)) {
			unit->sequential = spec->Expr;
		}
		else if (io_spec_is_name ("DIRECT", name, "CHARACTER", spec, unit->direct)) {
			unit->direct = spec->Expr;
		}
		else if (io_spec_is_name ("FORMATTED", name, "CHARACTER", spec, unit->formatted)) {
			unit->formatted = spec->Expr;
		}
		else if (io_spec_is_name ("FORM", name, "CHARACTER", spec, unit->form)) {
			unit->form = spec->Expr;
		}
		else if (io_spec_is_name ("UNFORMATTED", name, "CHARACTER", spec, unit->unformatted)) {
			unit->unformatted = spec->Expr;
		}
		else if (io_spec_is_name ("RECL", name, "INTEGER", spec, unit->recl)) {
			unit->recl = spec->Expr;
		}
		else if (io_spec_is_name ("NEXTREC", name, "INTEGER", spec, unit->nextrec)) {
			unit->nextrec = spec->Expr;
		}
		else if (io_spec_is_name ("BLANK", name, "CHARACTER", spec, unit->blank)) {
			unit->blank = spec->Expr;
		}

/*!!! Add the F90 specifiers */

		else {
			sprintf (serr, "invalid specifier in INQUIRE statement: %s", name);
			fatal (serr);
		}

		if (first)
			first = 0;
	}

	if (unit->unit == NN && unit->file == NN)
		fatal ("either the UNIT or FILE specifier is required in an INQUIRE statement");
	
	return root;
}

/***************************** W R I T E **********************************/

write_InquireStmt (node)
Node *node;
{
	if (flag_cubix)
		cubix_write_InquireStmt (node);
	else
		write_pass_childs (node);
}
